(* Legacy set of derivations

   license: GNU Lesser General Public License Version 2.1 or later
   ------------------------------------------------------------------------- *)

From elpi.apps Require Export derive.
From elpi.apps Require Export
  derive.projK
  derive.isK
  derive.eq
  derive.eqK
  derive.bcongr
  derive.eqOK
.
Elpi Typecheck derive.
